//
// @description: Example applications for Impact Acquire
// @copyright: Copyright (C) 2014 - 2021 Balluff GmbH
// @authors: APIs and drivers development team at Balluff GmbH
// @initial date: 2014-11-03
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice (including the next paragraph) shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,i
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

//-----------------------------------------------------------------------------
#include "Timeout.h"
#if defined(_WIN32) || defined(WIN32) || defined(__WIN32__)
#   include <windows.h>
#else
#   include <unistd.h>
#endif // #if defined(_WIN32) || defined(WIN32) || defined(__WIN32__)

//-----------------------------------------------------------------------------
CTimeout::CTimeout( unsigned long timeout_ms )
//-----------------------------------------------------------------------------
{
    Start( timeout_ms );
}

//-----------------------------------------------------------------------------
void CTimeout::Start( const unsigned long timeout_ms )
//-----------------------------------------------------------------------------
{
    start_ = clock();
    elapse_period_ = timeout_ms * CLOCKS_PER_SEC / 1000;
    if( elapse_period_ == 0 )
    {
        elapse_period_ = 1;
    }
    end_ = start_ + elapse_period_;
}

//-----------------------------------------------------------------------------
unsigned char CTimeout::Elapsed( void ) const
//-----------------------------------------------------------------------------
{
    return ( end_ < clock() );
}

//-----------------------------------------------------------------------------
void CTimeout::SleepMS( const unsigned long ms ) const
//-----------------------------------------------------------------------------
{
#if defined(_WIN32) || defined(WIN32) || defined(__WIN32__)
    Sleep( ms );
#else
    usleep( ms * 1000 );
#endif // #if defined(_WIN32) || defined(WIN32) || defined(__WIN32__)
}

//-----------------------------------------------------------------------------
unsigned long CTimeout::Remain( void ) const
//-----------------------------------------------------------------------------
{
    return Elapsed() ? 0 : ( ( clock() - end_ ) * 1000 ) / CLOCKS_PER_SEC;
}
