//
// @description: Example applications for Impact Acquire
// @copyright: Copyright (C) 2021 Balluff GmbH
// @authors: APIs and drivers development team at Balluff GmbH
// @initial date: 2021-07-30
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice (including the next paragraph) shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,i
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

#include "CheckedListCtrl.h"

//=============================================================================
//=================== Implementation CheckedListCtrl ==========================
//=============================================================================
BEGIN_EVENT_TABLE( CheckedListCtrl, wxListCtrl )
    EVT_LEFT_DOWN( CheckedListCtrl::OnMouseEvent )
END_EVENT_TABLE()

//-----------------------------------------------------------------------------
void CheckedListCtrl::OnMouseEvent( wxMouseEvent& event )
//-----------------------------------------------------------------------------
{
    if( event.LeftDown() )
    {
        int flags;
        long item = HitTest( event.GetPosition(), flags );
        if( item > -1 && ( flags & wxLIST_HITTEST_ONITEMICON ) )
        {
            SetChecked( item, !IsChecked( item ) );
            return;
        }
    }
    event.Skip();
}

//-----------------------------------------------------------------------------
bool CheckedListCtrl::IsChecked( long item ) const
//-----------------------------------------------------------------------------
{
    wxListItem info;
    info.m_mask = wxLIST_MASK_IMAGE;
    info.m_itemId = item;

    if( GetItem( info ) )
    {
        return ( info.m_image == 1 );
    }
    return false;
}

//-----------------------------------------------------------------------------
void CheckedListCtrl::SetChecked( long item, bool checked )
//-----------------------------------------------------------------------------
{
    SetItemImage( item, ( checked ? 1 : 0 ), -1 );
}
