//
// @description: Example applications for Impact Acquire
// @copyright: Copyright (C) 2007 - 2023 Balluff GmbH
// @authors: APIs and drivers development team at Balluff GmbH
// @initial date: 2007-09-04
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice (including the next paragraph) shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,i
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

//-----------------------------------------------------------------------------
#include <apps/Common/wxAbstraction.h>
#include "DeviceConfigureFrame.h"
#include "DeviceHandler.h"
#include <string>

#include <apps/Common/wxIncludePrologue.h>
#include "wx/numdlg.h"
#include <apps/Common/wxIncludeEpilogue.h>

using namespace std;

//-----------------------------------------------------------------------------
bool ShowMessageToUser( DeviceConfigureFrame* pParent, const wxString& caption, const wxString& msg, bool boSilentMode, long style, bool boError )
//-----------------------------------------------------------------------------
{
    bool boResult = false;
    if( boSilentMode )
    {
        if( pParent )
        {
            if( boError )
            {
                pParent->WriteErrorMessage( wxString::Format( wxT( "%s: %s\n" ), caption.c_str(), msg.c_str() ) );
            }
            else
            {
                pParent->WriteLogMessage( wxString::Format( wxT( "%s: %s\n" ), caption.c_str(), msg.c_str() ) );
            }
        }
        boResult = true;
    }
    else
    {
        wxMessageDialog dlg( pParent, msg, caption, style );
        switch( dlg.ShowModal() )
        {
        case wxID_OK:
        case wxID_YES:
        case wxID_APPLY:
        case wxID_YESTOALL:
            boResult = true;
            break;
        default:
            break;
        }
    }
    return boResult;
}

//=============================================================================
//======================== Implementation DeviceHandler =======================
//=============================================================================
//-----------------------------------------------------------------------------
bool DeviceHandler::GetIDFromUser( long& newID, const long minValue, const long maxValue ) const
//-----------------------------------------------------------------------------
{
    const string devSerial( pDev_->serial.read() );
    const string devProduct( pDev_->deviceModelName.read() );
    if( pParent_ )
    {
        pParent_->WriteLogMessage( wxString::Format( wxT( "Trying to set new ID for %s(%s). Current ID: %d\n" ), ConvertedString( devSerial ).c_str(), ConvertedString( devProduct ).c_str(), pDev_->deviceID.read() ) );
    }
    newID = wxGetNumberFromUser( wxString::Format( wxT( "Enter the new ID for device %s(%s).\nMake sure that no other device of the same family already owns this new ID.\n" ), ConvertedString( devSerial ).c_str(), ConvertedString( devProduct ).c_str() ),
                                 wxT( "New ID:" ),
                                 wxString::Format( wxT( "Set New ID For %s(%s)" ), ConvertedString( devSerial ).c_str(), ConvertedString( pDev_->deviceFamilyName.read() ).c_str() ),
                                 0,          // start
                                 minValue,   // min
                                 maxValue,   // max
                                 pParent_ );
    return true;
}

//-----------------------------------------------------------------------------
int DeviceHandler::UpdateFirmware( bool /*boSilentMode*/, bool /*boPersistentUserSets*/, bool /*boUseOnlineArchive*/ )
//-----------------------------------------------------------------------------
{
    if( pParent_ )
    {
        pParent_->WriteErrorMessage( wxString::Format( wxT( "Device %s doesn't implement firmware updates\n" ), ConvertedString( pDev_->serial.read() ).c_str() ) );
    }
    return urFeatureUnsupported;
}
