/*
 * @description: Example applications for Impact Acquire
 * @copyright: Copyright (C) 2007 - 2021 Balluff GmbH
 * @authors: APIs and drivers development team at Balluff GmbH
 * @initial date: 2007-09-04
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next paragraph) shall be included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

//-----------------------------------------------------------------------------
#ifndef DeviceHandlerBlueFOXH
#define DeviceHandlerBlueFOXH DeviceHandlerBlueFOXH
//-----------------------------------------------------------------------------
#include "DeviceHandler.h"

//-----------------------------------------------------------------------------
class DeviceHandlerBlueFOX : public DeviceHandler
//-----------------------------------------------------------------------------
{
public:
    DeviceHandlerBlueFOX( mvIMPACT::acquire::Device* pDev ) : DeviceHandler( pDev, true ) {}
    static DeviceHandler* Create( mvIMPACT::acquire::Device* pDev )
    {
        return new DeviceHandlerBlueFOX( pDev );
    }
    virtual bool SupportsFirmwareUpdate( void ) const
    {
        return true;
    }
    virtual bool SupportsFirmwareUpdateFromMVU( void ) const
    {
        return false;
    }
    virtual int GetLatestLocalFirmwareVersion( Version& latestFirmwareVersion ) const;
    virtual const wxString GetAdditionalUpdateInformation( void ) const
    {
        return wxString( wxT( "Firmware updates are part of the installed driver library. So updating the driver might also present a more recent firmware relase here!" ) );
    }
    virtual wxString GetStringRepresentationOfFirmwareVersion( const Version& version ) const
    {
        return wxString::Format( wxT( "%ld" ), version.major_ );
    }
    virtual int UpdateFirmware( bool boSilentMode, bool boPersistentUserSets, bool boUseOnlineArchive );
};

#endif // DeviceHandlerBlueFOXH
